from service.image_utils import *
from dbnet_crnn.image_text import image_text


def operation_morphology(img, operation_type, k):
    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (k, k))
    if operation_type == 'open':
        result = cv2.morphologyEx(img, cv2.MORPH_OPEN, kernel)
    else:
        result = cv2.morphologyEx(img, cv2.MORPH_CLOSE, kernel)
    return result


def get_text_contour(img, contour):
    k = 30
    img_h = img.shape[0]
    img_w = img.shape[1]
    contour_x1 = contour[0][0]
    contour_y1 = contour[0][1]
    contour_x2 = contour[2][0]
    contour_y2 = contour[2][1]
    x1 = contour_x1 - k if contour_x1 - k > 0 else contour_x1
    y1 = contour_y1 - k if contour_y1 - k > 0 else contour_y1
    x2 = contour_x2 + k if contour_x2 + k < img_w else img_w
    y2 = contour_y2 + k if contour_y2 + k < img_h else img_h
    text_contour = numpy.array([[x1, y1], [x2, y1], [x2, y2], [x1, y2]])
    return text_contour


def get_text_roi(img):
    img = cv2.imread('capture/'+img)
    img_blur = cv2.GaussianBlur(img, (3, 3), 0)
    gray = cv2.cvtColor(img_blur, cv2.COLOR_BGR2GRAY)
    binary = cv2.adaptiveThreshold(gray, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 5, 5)
    binary_open = operation_morphology(binary, 'open', 30)
    binary_open = cv2.GaussianBlur(binary_open, (5, 5), 0)
    contours = get_rectangle_contours(binary_open)
    img_roi = gray
    roi_text_list = []
    for contour in contours:
        contour = get_text_contour(img_roi, contour)
        roi = get_roi_image(img_roi, contour)
        roi = cv2.adaptiveThreshold(roi, 255, cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY, 7, 5)
        roi = cv2.cvtColor(roi, cv2.COLOR_GRAY2RGB)
        text = get_image_text(roi).replace(' ', '')
        if len(text) > 0:
            roi_text_list.append({'rect': contour.tolist(), 'pos': get_center_pos(contour), 'text': text})

    result = {
        'roi_text': roi_text_list,
        'img_shape': img.shape
    }
    return result


def get_image_text(image):
    img = cv2.imread('capture/' + image)
    h, w, _ = img.shape
    log_size = int(0.9*h)
    result = {
        'roi_text': image_text.get_text(img, log_size),
        'img_shape': img.shape
    }
    return result




__title__ = 'vision-ui'
__description__ = 'A line-oriented image diff algorithm.'
__url__ = 'https://github.com/Meituan-Dianping/vision-ui'
__version__ = '2.1.2'
__author__ = 'tech@meituan.com'
__author_email__ = ' tech@meituan.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Meituan-Dianping'
__cake__ = u'\u2728 \U0001f370 \u2728'


import cv2
import numpy


def merge_rectangle_contours(rectangle_contours):
    merged_contours = [rectangle_contours[0]]
    for rec in rectangle_contours[1:]:
        for i in range(len(merged_contours)):
            x_min = rec[0][0]
            y_min = rec[0][1]
            x_max = rec[2][0]
            y_max = rec[2][1]
            merged_x_min = merged_contours[i][0][0]
            merged_y_min = merged_contours[i][0][1]
            merged_x_max = merged_contours[i][2][0]
            merged_y_max = merged_contours[i][2][1]
            if x_min >= merged_x_min and y_min >= merged_y_min and x_max <= merged_x_max and y_max <= merged_y_max:
                break
            else:
                if i == len(merged_contours)-1:
                    merged_contours.append(rec)
    return merged_contours


def get_image_text(img, engine='cnocr'):
    text = 'cnocr'
    return text


def contour_area_filter(binary, contours, thresh=1500):
    rectangle_contours =[]
    h, w = binary.shape
    for contour in contours:
        if thresh < cv2.contourArea(contour) < 0.2*h*w:
            rectangle_contours.append(contour)
    return rectangle_contours


def get_roi_image(img, rectangle_contour):
    roi_image = img[rectangle_contour[0][1]:rectangle_contour[2][1],
                    rectangle_contour[0][0]:rectangle_contour[1][0]]
    return roi_image


def get_pop_v(image):
    """
    calculate value if a pop window exit
    :param image: image path
    :return: mean of v channel
    """
    img = cv2.imread('capture/'+image)
    img_hsv = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    h, s, v = cv2.split(img_hsv)
    return numpy.mean(v)


def get_rectangle_contours(binary):
    _, contours, _ = cv2.findContours(binary, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
    rectangle_contours = []
    for counter in contours:
        x, y, w, h = cv2.boundingRect(counter)
        cnt = numpy.array([[x, y], [x + w, y], [x + w, y + h], [x, y + h]])
        rectangle_contours.append(cnt)
    rectangle_contours = sorted(rectangle_contours, key=cv2.contourArea, reverse=True)[:100]
    rectangle_contours = contour_area_filter(binary, rectangle_contours)
    rectangle_contours = merge_rectangle_contours(rectangle_contours)
    return rectangle_contours


def get_center_pos(contour):
    x = int((contour[0][0]+contour[1][0])/2)
    y = int((contour[1][1]+contour[2][1])/2)
    return [x, y]


def get_label_pos(contour):
    center = get_center_pos(contour)
    x = int((int((center[0]+contour[2][0])/2)+contour[2][0])/2)
    y = int((int((center[1]+contour[2][1])/2)+contour[2][1])/2)
    return [x, y]


def draw_contours(img, contours, color="info"):
    if color == "info":
        cv2.drawContours(img, contours, -1, (255, 145, 30), 3)



# encoding: utf-8
import io
import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

about = {}
here = os.path.abspath(os.path.dirname(__file__))
with io.open(os.path.join(here, 'service', '__about__.py'), encoding='utf-8') as f:
    exec(f.read(), about)

with io.open("README.md", encoding='utf-8') as f:
    long_description = f.read()

install_requires = [
    "opencv-python==4.2.0.32",
    "numpy==1.19.1",
    "Flask==1.0.2",
    "Flask-Cors==3.0.7",
    "pillow==6.2.0",
    "paddlepaddle==1.8.4",
    "gunicorn==20.0.4",
    "onnxruntime==1.4.0",
    "pyclipper==1.2.0",
    "shapely==1.7.0"
]


class UploadCommand(Command):
    """ Build and publish this package.
        Support setup.py upload. Copied from requests_html.
    """

    user_options = []

    @staticmethod
    def status(s):
        """Prints things in green color."""
        print("\033[0;32m{0}\033[0m".format(s))

    def initialize_options(self):
        """ override
        """
        pass

    def finalize_options(self):
        """ override
        """
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        # self.status('Publishing git tags…')
        # os.system('git tag v{0}'.format(about['__version__']))
        # os.system('git push --tags')

        sys.exit()


setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    license=about['__license__'],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, <4',
    # packages=['dbnet_crnn','service'],
    packages=find_packages(),

    package_data={
        '': ["README.md"],
        'dbnet_crnn/model/det': ['dbnet_crnn/model/det/model', 'dbnet_crnn/model/det/params'],
        'dbnet_crnn/model/rec': ['dbnet_crnn/model/rec/model', 'dbnet_crnn/model/rec/params'],
        'dbnet_crnn/ppocr/utils': ['dbnet_crnn/ppocr/utils/keys.txt'],
    },
    data_files=[('dbnet_crnn/model/det', ['dbnet_crnn/model/det/model', 'dbnet_crnn/model/det/params']),
                ('dbnet_crnn/model/rec', ['dbnet_crnn/model/rec/model', 'dbnet_crnn/model/rec/params']),
                ('dbnet_crnn/ppocr/utils', ['dbnet_crnn/ppocr/utils/keys.txt'])],
    keywords='A line-oriented image diff algorithm',
    install_requires=install_requires,
    extras_require={},
    classifiers=[
        "Development Status :: 3 - Alpha",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    # entry_points={
    #     'console_scripts': [
    #         'ate=httprunner.cli:main_hrun',
    #         'httprunner=httprunner.cli:main_hrun',
    #         'hrun=httprunner.cli:main_hrun',
    #         'locusts=httprunner.cli:main_locust'
    #     ]
    # },
    # $ setup.py upload support.
    cmdclass={
        'upload': UploadCommand
    }
)
